#include <OneWire.h>
#include <LCD4Bit.h> 
//create object to control an LCD.  
//number of lines in display=1
LCD4Bit lcd = LCD4Bit(2);  //two line LCD, use 1 if you only have a 1 line LCD

// DS18S20 Temperature chip i/o
OneWire ds(4);
byte tempMAC[] = {0x28, 0x77, 0x19, 0x16, 0x2, 0x0, 0x0, 0xF2}; // address of one wire temp sensor


void setup(void) {
  // start serial port
  Serial.begin(9600);
  lcd.init();
  delay(1000);
  lcd.clear();
  lcd.printIn("Current Temp: ");
  delay(1000);
}

void loop(void) {
  int HighByte, LowByte, TReading, SignBit, Tc_100, Whole, Fract;
  char charTemp[6];

  delay(9000); // delay 10 (9 + the 1 second below) seconds for temp readings
  byte i;
  byte present = 0;
  byte data[12];
  byte addr[8];

  ds.reset();
  ds.select(tempMAC);
  ds.write(0x44,1);         // start conversion, with parasite power on at the end

  delay(1000);     // maybe 750ms is enough, maybe not
  present = ds.reset();
  ds.select(tempMAC);    
  ds.write(0xBE);         // Read Scratchpad

  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
  }
  LowByte = data[0];
  HighByte = data[1];
  TReading = (HighByte << 8) + LowByte;
  SignBit = TReading & 0x8000;  // test most sig bit
  if (SignBit) // negative
  {
    TReading = (TReading ^ 0xffff) + 1; // 2's comp
  }
  Tc_100 = (6 * TReading) + TReading / 4;    // multiply by (100 * 0.0625) or 6.25

  Whole = ((Tc_100 * 1.8) + 3200) / 100;  // separate off the whole and fractional portions and convert from C to F
  Fract = Tc_100 % 100;
  
  lcd.cursorTo(2, 0);  //line=2, x=0.
  if (SignBit) // If its negative
  {
     Serial.print("-");
     lcd.printIn("-");
  }
  Serial.print(fTemp);
  Serial.print(" F");
  Serial.println();

  lcd.printIn(itoa(Whole, charTemp, 10));
  lcd.printIn(".");
  lcd.printIn(itoa(Fract, charTemp, 10));
  lcd.printIn("F   "); // we put a few spaces here to deal with temps with no fract
}
